%
%	MAINM.M
%
%	This program uses array_lm.m (linear array) or array_cm.m (circular
%  array and allows the use of doa estimation methods for adaptive source 
%  location. The program first read from the mentioned file array_lm/array_cm.m
%  the data corresponding to the aperture and the source parameters.
%	There are two methods of scanning 1d and 2d. The plot for 2d is a
%  polar diagram where sinus of the elevation is the ro and the azimuth is
%  the theta variable on the plot. This version of 2d runs very slow due to
%  the computational burden motivated in graphics and 2d estimates.
%	Next, the program ask for a method for doa estimation, including
%  periodogram or phased array, maximum likelihood (Capon), normalised
%  maximum likelihood (Lagunas), music, adaptive em (A.I. Perez and M.A.
%  Lagunas).
%	The new method is the so-called em. It is done by means of two 
%  paralell blocks able for single source estimation, using either phased
%  or nmlm, being the estimate of each block used sequentially to block 
%  the corresponding doa in the other one. This is iterated until, either
%  the two doa estimates remain or until the number of iterations arise
%  to "litt". After the iterations, in the em fashion, ends, two kalman
%  trackers predict the new doas for the next time the estimate have to
%  be updated. The tracker is sensitive to the radial velocity of the
%  sources.
%	The estimates are feed with the current snapshot covariance. This
%  is done every "nite" snapshots. The covariance matrix is updated with
%  this time constant by means parameter "beta". In order to produce reliable
%  estimates, the covariance matrix is not provided for adaptive processing
%  until the number of collected snapshots reach 10 times the number of 
%  sensors.
%
%  Miguel Angel Lagunas				January 1996
%
%........................................................................
%________________________________________________________________________
clear all;clf;clc;
% read array and source parameters
k=menu('Select aperture','uniform linear array','2d  array');
if k==1;
        array_lm;
elseif k==2;
        array_cm;
end;
% change degrees per radians in data
pi=3.14159265;twopi=2*pi;ii=sqrt(-1);fiser=fisen*(pi/180);
souelr=souele*(pi/180);souelrv=souelev*pi/180;
souazr=souazi*(pi/180);souazrv=souaziv*pi/180;soua=souazr;soue=souelr;
% change dB per levels in source strenght
for i=1:nsou;
        soule(i)=sqrt(10^(soulev(i)/10));
end;
%_____________________________________________________________________________
% select doa estimation method and 1d(Linear aperture) or 2d scanning (planar)
km=menu('Estimation method','Phased','MLM','NML','Music','EM-nml'...
,'End of the program');
if km<6;
  di=menu('Scanning mode','Elevation only','Elevation and azimuth');
else
  break;
end;
if km==5;
  if nsou>2;
    disp('ERROR:The em is not prepared in this program to track more');
    disp('than TWO sources. Modify sou_lm.m before continuing');break;
  end;
end;
%_____________________________________________________________________________
% set the number of snapshots for updating the doa estimate. this means that
% every 'nite' snapshots the doa's will be estimated and presented en the final
% plot.
nite=10;nit=nite;
% the parameter 'beta' is used to estimate the array covariance from the 
% received snapshots. the time constant is set equal to nite
beta=(nite-1)/(nite);
% size of the covariance matrix
cov=zeros(length(d),length(d));
%______________________________________________________________________________
% specific data for the em algorithm
% number of iterations for the e-m loop
if di==1;
  litt=10;
elseif di==2;
  litt=2;
end;
% after 'litt' iterations maximum, the em pass doa and radial velocity to Kalman
% trackers. next the initial parameters of the trackers are set
km1=0;km0=0;est1=[0;0];est0=est1;
if di==2;
  est1=[0;0;0;0];est0=est1;
end;
%_______________________________________________________________________________
% compute bounds for graphics of actual source evolution
souei=souele;souef=souele+nsnap*souelev;souai=souazi;souaf=souazi+nsnap*souaziv;
aa=round(min([souei souef]))-5;bb=round(max([souei souef]))+5;clf;
aaa=round(min([souai souaf]))-5;bbb=round(max([souai souaf]))+5;
% scanning precission and check bounds
if di==1;
  tetap=0.2;azip=0;
elseif di==2;
  tetap=1.5;azip=3;
end;
teta=aa:tetap:bb;tetar=teta*pi/180;ndib=length(teta);
azir=aaa:azip:bbb;azirr=azir*pi/180;ndibz=length(azir);
s=['Elevation Field of view from  ' num2str(teta(1)) ' up to  ' num2str(teta(length(teta)))];
disp(s);s=['Scanning precission in elevation of  ' num2str(teta(2)-teta(1)) ' degrees'];
disp(s);ele(1,1)=teta(1);ele(2,1)=teta(length(teta));
if di==2;
  s=['Azimuth field of view from ' num2str(azir(1)) ' up to  ' num2str(azir(length(azir)))];
  disp(s);s=['Scanning precission in azimuth of ' num2str(azir(2)-azir(1)) ' degrees'];
  disp(s);azir0=azir(1);azir1=azir(length(azir));azir0r=azirr(1);azir1r=azirr(length(azir));
end;
teta0r=tetar(1);teta1r=tetar(length(tetar));moona=[];moone=[];fi=0;
disp('---------------------------------------------------------------------------')
clear aa aaa bb bbb;
disp('Press return to continue');pause;
%_________________________________________________________________________________
%  the covariance matrix is initialised  with 10*length(sensors) snapshots before
%  any estimation starts
if di==1;
  cas=10;
elseif di==2;
  cas=20;
end;
for im=1:cas*nsen;
  souele=souele+souelev;
  for i=1:nsou;
    dsou(:,i)=(2*pi*fc/c)*d'.*cos(fiser'-soua(i)*ones(nsen,1))*sin(soue(i));
  end;
  soua=soua+souazrv;soue=soue+souelrv;dsou=exp(-ii*dsou);aux=diag(soule);
  souwav=aux*exp(ii*(fresou'*im));
  xsna=dsou*souwav+randn(nsen,1)-ii*randn(nsen,1);
  cov=cov*beta+(1-beta)*xsna*xsna';
end;icov=inv(cov);clear xsna aux;
%__________________________________________________________________________________
%  the bandwidth of the array for any steering direction is plot. to activate this
%  feature set icon equal to one. it will take a few minutes
icon=0;
if icon==1;
  disp('Computing the array bandwidth versus steering. It will take some minutes')
  disp('Press return after plot');tetau=-90:0.5:90;tetaur=tetau*pi/180;
  s=ii*(2*pi*fc/c)*d'.*cos(fiser')*sin(tetaur);s=exp(-s);
  for ji=1:length(tetaur)-1;
    suso=ii*(2*pi*fc/c)*d'.*cos(fiser')*sin(tetaur(ji));suso=exp(-suso);
    per(ji,:)=20*log10(abs(suso'*s));tuso=max(per(ji,:));perb=tuso-3;
    for ij=2:length(tetaur)-1;
      if per(ji,ij)<perb;
        if per(ji,ij+1)>=perb;
          luso(1)=tetau(ij);
        end;
      end;
      if per(ji,ij)>perb;
        if per(ji,ij+1)<=perb;
          luso(2)=tetau(ij);
        end;
      end;
    end;
    buss(ji)=abs(luso(2)-luso(1));
    if buss(ji)>90;
      buss(ji)=180-buss(ji);
    end;
  end;clf;xx=tetau(1:length(tetau)-1);xdib=[min(xx) max(xx) min(buss) max(buss)];
  axis(xdib);plot(xx,buss,'y');grid;xlabel('steering angle ()');
  ylabel('bandwidth ()');title('Array bandwidth');figure;
end;
%_________________________________________________________________________________
% main loop
im0=1;
for im=cas*nsen+1:nsnap
%   actual position of the sources
  souele=souele+souelev;
%   compute snapshots
  for i=1:nsou;
        dsou(:,i)=(2*pi*fc/c)*d'.*cos(fiser'-soua(i)*ones(nsen,1));
        dsou(:,i)=dsou(:,i)*sin(soue(i));
  end;
  soua=soua+souazrv;soue=soue+souelrv;dsou=exp(-ii*dsou);
  aux=diag(soule);souwav=aux*exp(ii*(fresou'*im));
  xsna=dsou*souwav+randn(nsen,1)-ii*randn(nsen,1);
%    update covariance estimate
  cov=cov*beta+(1-beta)*xsna*xsna';clear aux xsna;
%
% Metodos de DOA
%
  nit=nit-1;
%    update every nite snapshots the estimates
  if nit==0;
    if di==1;clf;
      s=['Snapshot # ' num2str(im) ];disp(s);
%      plot actual source location at this instant
      xdib=[min(teta) max(teta) min(soulev-10) max(soulev+10)];axis(xdib);
      xlabel('Elevation in degrees');ylabel('dB');
      grid;hold on;au=min(soulev)-15;bu=max(soulev)+10;fi=fi+1;
      for i=1:ndib;
        mlma(i)=au;mlmb(i)=0;
        for j=1:nsou;
                if teta(i)<=souele(j);
                        if teta(i+1)>souele(j);
				elea(j,round(im/nite))=teta(i);
                                mlma(i)=soulev(j);
				mlmb(i)=bu;
                        end;
                end;
        end;
      end;
      plot(teta,mlma,'r');moone=[moone mlma'];
      nit=nite;
    elseif di==2;
      s=['Snapshot # ' num2str(im) ];disp(s);
      if im0==1;
        clf;polar(0,1,'.');hold on;
        title('slowness/azimuth plot, actual blue, estimated green');
      end;hold on;
      for ir=1:nsou;
        roa(ir,im0)=sin(soue(ir));azia(ir,im0)=soua(ir);
        polar(azia(ir,im0),roa(ir,im0),'bo');hold on;
      end;nit=nite;im0=im0+1;
    end;
%_____________________________________________________________________________
%		Phased array method
    if km==1;
        pmm;
%______________________________________________________________________________
%		MLM method
    elseif km==2;
        mlmm;
%_______________________________________________________________________________
%		NMLM method
    elseif km==3;
        nmlmm;
%________________________________________________________________________________
%		Music method
    elseif km==4;
        musm;
%________________________________________________________________________________
%		EM algorithm
    elseif km==5;
        emm;
    end;
%		no more methods
    if km==6;
      disp('end of the program');break;
    end;
  end;
end;

%  end main loop
if km<6;
  if di==1;
%___________________________________________________________________________________
% dibujos 2d
%       contour plot
    pause(0.1);hold off;clf;au=min(min(moona));bu=max(max(moona));fi=1:1:fi;
% number of levels for contour plot, include vc for labels
    nniv=10;vc=au:(bu-au)/nniv:bu;
    cs=contour(fi,teta,moona,vc);
%clabel(cs);
    hold on;contour(fi,teta,moone,vc);xlabel('scans');ylabel('Elevation in degrees');
    title('DOA Estimation');pause;hold off;clf;
%___________________________________________________________________________________
%      e-d plot
    view(-45,65);
    mesh(fi,teta,moona);title('DOA Estimation');xlabel('scans');
    ylabel('Elevation in degrees');hold on;mesh(fi,teta,moone);
    view(-45,65);pause;hold off;clf;
%___________________________________________________________________________________
%      tracking plot 
    xdib=[1 nsnap/nite min(min(ele)) max(max(ele))];axis(xdib);
    xx=1:1:nsnap/nite;
    for j=1:nsou;
      plot(xx,ele(j,:),'r');
      if j==1;
        grid;hold on;
      end;
      plot(xx,elea(j,:),'b');
    end;
    xlabel('scans'):ylabel('DOA detected (red)');title('Tracking plot (blue=actual)')
  elseif di==2;
    if km==5;
      figure;ndib=length(azia(1,:));il0=length(azie(1,:));il=il0-ndib+1;
      azie=azie*180/pi;roe=asin(roe)*180/pi;
      azia=azia*180/pi;roa=asin(roa)*180/pi;
      xdib=[1 ndib min(min(azie)) max(max(azie))];
      clf;axis(xdib);xx=il:1:length(azie);plot(xx,azie(1,il:il0),'g');grid;hold on;
      plot(xx,azie(2,il:il0),'g');plot(xx,azia(1,:),'b');plot(xx,azia(2,:),'b');
      xlabel('snapshots');ylabel('azimuth in degrees');title('Azimuth tracking plot')
      pause(0.1);figure;
      xdib=[1 ndib min(min(roe)) max(max(roe))];
      clf;axis(xdib);plot(xx,roe(1,il:il0),'g');grid;hold on;
      plot(xx,roe(2,il:il0),'g');plot(xx,roa(1,:),'b');plot(xx,roa(2,:),'b');
      xlabel('snapshots');ylabel('elevation in degrees');
      title('Elevation tracking plot');pause(0.1);
    end;
  end;
end;
%___________________________________________________________________________________
%  end of the program
%...................................................................................
